unit principale;

interface

uses
  System.SysUtils, System.Types, System.UITypes, System.Classes, System.Variants,
  FMX.Types, FMX.Controls, FMX.Forms, FMX.Graphics, FMX.Dialogs,
  FMX.Objects, FMX.StdCtrls, FMX.Layouts, FMX.Effects, FMX.Filter.Effects,
  FMX.Controls.Presentation, FMX.Media, System.Sensors,
  System.Sensors.Components
  {$IFDEF Android}
  , Androidapi.JNI.App,
  Androidapi.JNI.GraphicsContentViewText,
  Androidapi.Helpers
  {$ENDIF}
  ;

type
  TForm1 = class(TForm)
    BackgroundRect: TRectangle;
    BackgroundImage: TImage;
    HeaderToolBar: TToolBar;
    HeaderBackgroundRect: TRectangle;
    HeaderLabel: TLabel;
    VertScrollBox1: TVertScrollBox;
    layTorche: TLayout;
    PrivacyLabel: TLabel;
    swTorche: TSwitch;
    PrivacyImage: TImage;
    FillRGBEffect2: TFillRGBEffect;
    layActiveVeilleuse: TLayout;
    CloudLabel: TLabel;
    swVeilleuse: TSwitch;
    layActiveGPS: TLayout;
    PremiumLabel: TLabel;
    swGPS: TSwitch;
    PremiumImage: TImage;
    FillRGBEffect7: TFillRGBEffect;
    layActiveMeteo: TLayout;
    PaymentLabel: TLabel;
    swMeteo: TSwitch;
    PaymentImage: TImage;
    FillRGBEffect8: TFillRGBEffect;
    StyleBook1: TStyleBook;
    Camera: TCameraComponent;
    Image1: TImage;
    FillRGBEffect1: TFillRGBEffect;
    layInfosGPS: TLayout;
    LocationSensor: TLocationSensor;
    layInfosMeteo: TLayout;
    veilleuse: TRectangle;
    lblLongitude: TLabel;
    lblLatitude: TLabel;
    lblAltitude: TLabel;
    Timer1: TTimer;
    lblTemperature: TLabel;
    lblPression: TLabel;
    lblHumidite: TLabel;
    procedure swVeilleuseSwitch(Sender: TObject);
    procedure swTorcheSwitch(Sender: TObject);
    procedure LocationSensorLocationChanged(Sender: TObject;
      const OldLocation, NewLocation: TLocationCoord2D);
    procedure swGPSSwitch(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure Timer1Timer(Sender: TObject);
    procedure swMeteoSwitch(Sender: TObject);
  private
    procedure SetFlashlightState(Active: Boolean);
    function DegDecToDMS(DegDec: Real): String;
    procedure CreateIfExists(ASensorCategory: TSensorCategory);
    procedure updateConditionsMeteo;
    { Dclarations prives }
  public
    { Dclarations publiques }
    FSensorTemperature, FSensorPression, FSensorHumidite: TCustomSensor;
  end;

const
  AllCat : TSensorCategories =
  [TSensorCategory.Location, TSensorCategory.Environmental, TSensorCategory.Motion,
  TSensorCategory.Orientation, TSensorCategory.Mechanical, TSensorCategory.Electrical,
  TSensorCategory.Biometric, TSensorCategory.Light, TSensorCategory.Scanner];
  cForm = '%3.2f';

var
  Form1: TForm1;

implementation

{$R *.fmx}

procedure TForm1.swGPSSwitch(Sender: TObject);
begin
  LocationSensor.Active := swGPS.IsChecked;
end;

procedure TForm1.swMeteoSwitch(Sender: TObject);
begin
  Timer1.Enabled := swMeteo.IsChecked;
end;

procedure TForm1.swTorcheSwitch(Sender: TObject);
begin
  SetFlashlightState(swTorche.IsChecked);
end;

procedure TForm1.swVeilleuseSwitch(Sender: TObject);
begin
  veilleuse.Visible := swVeilleuse.IsChecked;
end;

procedure TForm1.Timer1Timer(Sender: TObject);
begin
  updateConditionsMeteo;
end;

procedure TForm1.LocationSensorLocationChanged(Sender: TObject;
  const OldLocation, NewLocation: TLocationCoord2D);
var
  LSettings: TFormatSettings;
  LDecSeparator : Char;
begin
  LDecSeparator := FormatSettings.DecimalSeparator;
  LSettings := FormatSettings;
  try
    FormatSettings.DecimalSeparator := '.';
    lblLongitude.Text := '';
    lblLatitude.Text := '';
    lblAltitude.Text := '';
    // Affiche la position actuelle
    if NewLocation.Latitude > 0 then
      lblLatitude.Text := 'Latitude :  '+Format('%2.2f', [NewLocation.Latitude])+' Nord '+sLineBreak+'     en DMS : '+DegDecToDMS(NewLocation.Latitude)
    else
      lblLatitude.Text := 'Latitude :  '+Format('%2.2f', [NewLocation.Latitude])+' Sud '+sLineBreak+'     en DMS : '+DegDecToDMS(NewLocation.Latitude);

    if NewLocation.Longitude > 0 then
      lblLongitude.Text := 'Longitude :  '+Format('%2.2f', [NewLocation.Longitude])+' Est'+sLineBreak+'     en DMS : '+DegDecToDMS(NewLocation.Longitude)
    else
      lblLongitude.text := 'Longitude :  '+Format('%2.2f', [abs(NewLocation.Longitude)])+' Ouest'+sLineBreak+'     en DMS : '+DegDecToDMS(NewLocation.Longitude);

    lblAltitude.Text := 'Altitude :  '+format('%3.2f',[LocationSensor.Sensor.Altitude])+' m';

  finally
    FormatSettings.DecimalSeparator := LDecSeparator;
  end;
end;

// Permet de convertir les coordonnes GPS en degr au format Degrs, Minutes, Secondes
function TForm1.DegDecToDMS( DegDec : Real ):String;
var
  Tmp : real;
  S : string;
      function DeuxCar( S1 : String): String;
      begin
         if length(S1)=1 then Result := '0'+S1 else Result := S1;
      end;
begin
    S := '';
    S := abs(trunc(DegDec)).ToString;  // rcupration la partie entire
    Tmp := Frac(DegDec);   // renvoi la partie dcimale
    Tmp := Tmp * 60;
    S := S + '' + DeuxCar( abs(trunc(Tmp)).ToString);
    Tmp := Frac(Tmp)*60;
    S := S + chr(39) + DeuxCar( abs(trunc(Tmp)).ToString);
    Tmp := Frac(Tmp)*1000;
    S := S + '"' + abs(trunc(Tmp)).ToString;
    Result := S;
end;

procedure TForm1.FormCreate(Sender: TObject);
var
  LSensorCat : TSensorCategory ;
begin
  {$IFDEF Android}
  SharedActivity.getWindow.addFlags(TJWindowManager_LayoutParams.JavaClass.FLAG_KEEP_SCREEN_ON);
  {$ENDIF}
  FSensorTemperature := nil;
  FSensorPression := nil;
  FSensorHumidite := nil;

  TSensorManager.Current.Activate();
  for LSensorCat in AllCat do
    CreateIfExists(LSensorCat);
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
  if (FSensorTemperature <> nil) then FSensorTemperature.Stop;
  if (FSensorPression <> nil) then FSensorPression.Stop;
  if (FSensorHumidite <> nil) then FSensorHumidite.Stop;
end;

procedure TForm1.CreateIfExists(ASensorCategory: TSensorCategory);
var
  LSensorArray : TSensorArray;
  LSensor : TCustomSensor;
  ls : TCustomEnvironmentalSensor;
  LProp : TCustomEnvironmentalSensor.TProperty;
begin
  LSensorArray := TSensorManager.Current.GetSensorsByCategory(ASensorCategory);
  for LSensor in LSensorArray do
  begin
    if LSensor.Category = TSensorCategory.Environmental then
    begin
      ls := TCustomEnvironmentalSensor(LSensor);
      for LProp in ls.AvailableProperties do
      begin
        case LProp of
          TCustomEnvironmentalSensor.TProperty.Temperature:
            begin
              FSensorTemperature := LSensor;
              FSensorTemperature.Start;
            end;
          TCustomEnvironmentalSensor.TProperty.Pressure:
            begin
              FSensorPression := LSensor;
              FSensorPression.start;
            end;
          TCustomEnvironmentalSensor.TProperty.Humidity:
            begin
              FSensorHumidite := LSensor;
              FSensorHumidite.start;
            end;
        end;
      end;
    end;
  end;
end;

procedure TForm1.SetFlashlightState(Active : Boolean);
begin
  if Active then
  begin
    Camera.Active := True;
    Camera.TorchMode := TTorchMode.ModeOn;
  end else
  begin
    Camera.TorchMode := TTorchMode.ModeOff;
    Camera.Active := false;
  end;
end;

procedure TForm1.updateConditionsMeteo;
var
  ls : TCustomEnvironmentalSensor;
begin
  ls := TCustomEnvironmentalSensor(fSensorTemperature);
  if Assigned(ls) then
  begin
    lblTemperature.text := 'Temprature: ' + Format(cForm,[ls.Temperature])+' C';
  end;

  ls := TCustomEnvironmentalSensor(fSensorPression);
  if Assigned(ls) then
  begin
    lblPression.text := 'Pression: ' + Format(cForm,[ls.Pressure])+' hPa';
  end;

  ls := TCustomEnvironmentalSensor(fSensorHumidite);
  if Assigned(ls) then
  begin
    lblHumidite.text := 'Humidit: ' +Format(cForm,[ls.Humidity])+'%';
  end;
end;

end.
